<?php
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 *
 */
class Routee_Functions{

	const AUTH_MODE  = 'auth';
	const DATA_MODE  = 'data';
	const EVENT_MODE = 'event';

	/**
	 *
	 */
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'Routee_Menu' ) );

		add_action( 'wp_ajax_routee_login', array( $this, 'Routee_login' ) );
		add_action( 'wp_ajax_nopriv_routee_login', array( $this, 'Routee_login' ) );

		add_action( 'user_register', array( $this, 'Routee_NewUserRegister' ) );
		add_action( 'woocommerce_update_customer', array( $this, 'Routee_UserUpdate' ) , 10, 3 );

		add_action( 'transition_post_status', array( $this, 'Routee_ProductHook' ), 10, 3 );

		add_action( 'woocommerce_no_stock', array( $this, 'Routee_woocommerce_no_stock' ), 10, 1 );
		add_action( 'woocommerce_updated_product_stock', array( $this, 'Routee_woocommerce_updated_product_stock' ), 10, 1 );

		add_action( 'woocommerce_add_to_cart', array( $this, 'Routee_woocommerce_add_to_cart' ), 10, 3 );
		add_action( 'woocommerce_after_cart_item_quantity_update', array( $this, 'Routee_woocommerce_add_to_cart' ), 20, 4 );
		add_action( 'woocommerce_remove_cart_item', array( $this, 'Routee_remove_cart_items' ), 10, 2 );

		add_action( 'woocommerce_checkout_order_processed', array( $this, 'Routee_woocommerce_checkout_order' ), 10, 1 );
		add_action( 'woocommerce_order_status_changed', array( $this, 'Routee_order_update_callback' ), 10, 1 );
		add_action( 'woocommerce_order_status_completed', array( $this, 'Routee_woocommerce_order_status_completed' ), 10, 3 );
	}

	/**
	 * @return void
	 */
	public function Routee_Menu(){
		add_menu_page(
			__('Routee_Menu'),
			'Routee',
			'edit_pages',
			'Routee_Menu',
			array( $this, 'Routee_login_screen' )
		);
	}

	/**
	 * @return void
	 */
	public function Routee_login_screen(){
		$RouteeAuthorized = get_option( 'RouteeAuthorized' );
		if($RouteeAuthorized == '1'){
			include 'thank.php';
		}else{
			include 'login.php';
		}
	}

	/**
	 * @return void
	 */
	public function Routee_login() {
		$nonce = $_POST['_ajax_nonce'];

		if ( ! wp_verify_nonce( $nonce, 'ajax-nonce' ) )
			die ( 'Busted!');

		ob_clean();
		parse_str($_POST['form_data'],$data);

		$login_data = array(
			'username' => $data['routee_user'],
			'password' => $data['routee_password'],
			'source' => 'WooCommerce',
			'type' => 'E-COMMERCE',
			'version' => '1.0.0',
			'plugin_version' => ROUTEE_PLUGIN_VERSION,
			'domain' => get_site_url()
		);

		$result = Routee_http_call( self::AUTH_MODE, $login_data );

		if( $result['success']  && isset( $result['data']['uuid'] ) ){

			$uuid = $result['data']['uuid'];
			/* Save option in Wp */
			update_option( 'RouteeLoginUser', trim($data['routee_user']));
			update_option( 'RouteeLoginPswd', trim($data['routee_password']));
			update_option( 'RouteeLoginCode', trim($uuid));
			update_option( 'RouteeAuthorized', trim('1'));

			$flag = true;
			$msg = 'You are successfully authorized';
			/* Authentication process */
			global $wpdb;

			$querystr = "
			    SELECT $wpdb->posts.ID
			    FROM $wpdb->posts
			    WHERE $wpdb->posts.post_status = 'publish' 
			    AND $wpdb->posts.post_type = 'product'
			    ORDER BY $wpdb->posts.post_date ASC
			 ";

			$pageposts = $wpdb->get_results($querystr, OBJECT);
			//Added product mass data to send over routee in batch of 100 on 15th July 2020 as per WM-376
			$p = 0;
			$ptotal = 0;

			$total_products = count($pageposts);

			$mass_data_product = array(
				'uuid' => $uuid,
				'data' => array(
					array(
						'name' => 'Products',
						'description' => 'Shop Products'
					)
				)
			);
			//Ends here

			foreach ($pageposts as $key => $post) {
				$WC_Product = wc_get_product( $post->ID);

				//Added product mass data to send over routee in batch of 100 on 15th July 2020 as per WM-376
				$mass_data_product['data'][0]['object'][$p] =  RouteeProductArray($WC_Product);

				$p++;
				$ptotal++;

				if ($p == 100 || $ptotal == $total_products) {

					Routee_http_call( self::DATA_MODE, $mass_data_product );
					$p = 0;
					$mass_data_product['data'][0]['object'] = array();
				}
				//Ends here
			}


			/* Customers Synchronization */
			$cust_qry = "SELECT ID FROM ".$wpdb->prefix ."users ";

			$cust_res = $wpdb->get_results($cust_qry, OBJECT);
			$cust_ary = array();

			//Added customer mass data to send over routee in batch of 100 on 15th July 2020 as per WM-376
			$c = 0;
			$ctotal = 0;

			$total_customers = count($cust_res);

			$mass_data_customer = array(
				'uuid' => $uuid,
				'data' => array(
					array(
						'name' => 'Customers',
						'description' => 'Shop Customers'
					)
				)
			);
			//Ends here

			foreach ($cust_res as $key => $cust) {

				$mass_data_customer['data'][0]['object'][$c] = RouteeUserArray( $cust->ID );

				//Added customer mass data to send over routee in batch of 100 on 15th July 2020 as per WM-376
				$c++;
				$ctotal++;

				if ($c == 100 || $ctotal == $total_customers) {

					Routee_http_call( self::DATA_MODE, $mass_data_customer );

					$c = 0;
					$mass_data_customer['data'][0]['object'] = array();
				}
				//Ends here
			}

			/* Order synchronize*/
			$orderstr = "
			    SELECT $wpdb->posts.ID
			    FROM $wpdb->posts
			    WHERE $wpdb->posts.post_type = 'shop_order'
			    ORDER BY $wpdb->posts.post_date ASC
			 ";

			$order_res = $wpdb->get_results($orderstr, OBJECT);
			$order_ary = array();

			//Added order mass data to send over routee in batch of 100 on 9th July 2020 as per WM-376
			$o = 0;
			$ototal = 0;

			$total_orders = count($order_res);

			$mass_data_order = array(
				'uuid' => $uuid,
				'data' => array(
					array(
						'name' => 'Orders',
						'description' => 'Shop Orders'
					)
				)
			);
			//Ends here

			foreach ($order_res as $key => $order) {
				$order_id = $order->ID;
				//Added order mass data to send over routee in batch of 100 on 9th July 2020 as per WM-376
				$mass_data_order['data'][0]['object'][$o] =  RouteeOrderDetails( $order_id );
				//Ends here
				$order = wc_get_order( $order_id );
				$order_items = $order->get_items();

				foreach( $order_items as $product ) {
					$mass_data_order['data'][0]['object'][$o]['order_products'][] =array(
						'product_id' => $product['product_id'],
						'variation_id' => $product['variation_id'],
						'quantity'  =>  $product['qty']
					);
				}

				//Added order mass data to send over routee in batch of 100 on 9th July 2020 as per WM-376
				$o++;
				$ototal++;

				if ($o == 100 || $ototal == $total_orders) {
					Routee_http_call( self::DATA_MODE, $mass_data_order );
					$o = 0;
					$mass_data_order['data'][0]['object'] = array();
				}
				//Ends here
			}

		}else{
			$flag = false;
			$msg = isset( $result['data']['message'] ) ? $result['data']['message'] : "Some error occured, Please try again";
		}

		$resp = array(
			'flag' => $flag,
			'msg' => $msg,
		);
		wp_send_json($resp);
	}

	/**
	 * @param $user_id
	 *
	 * @return void
	 */
	public function Routee_NewUserRegister( $user_id ) {
		$post_data = RouteeUserArray($user_id);
		Routee_http_call( self::EVENT_MODE, $post_data,'CustomerAdd', $user_id);
	}

	/**
	 * @param $user_id
	 *
	 * @return void
	 */
	public function Routee_UserUpdate( $user_id ){
		$post_data = RouteeUserArray($user_id);
		Routee_http_call( self::EVENT_MODE, $post_data,'CustomerUpdate', $user_id );
	}

	/**
	 * @param $new_status
	 * @param $old_status
	 * @param $post
	 *
	 * @return void
	 */
	public function Routee_ProductHook( $new_status, $old_status, $post ){
		if ( $post->post_type != 'product' ) {
			return;
		}
		$product_id = $post->ID;

		switch(true) {

			case ( $old_status == 'auto-draft' && $new_status == 'publish' ) :
			case ( $old_status == 'draft' && $new_status == 'publish' ) :
			case ( $old_status == 'trash' && $new_status == 'publish' ) :
				$this->Routee_new_product( $product_id );
			break;

			case ( $old_status == 'publish' && $new_status == 'publish' ) :
				$this->Routee_update_product( $product_id );
			break;

			case ( $old_status == 'publish' && $new_status == 'trash' ) :
				$this->Routee_trash_product( $product_id );
			break;

		}
	}

	/**
	 * @param $product_id
	 *
	 * @return void
	 */
	public function Routee_new_product( $product_id ) {
		$product = wc_get_product( $product_id );
		$post_data = RouteeProductArray( $product );
		Routee_http_call( self::EVENT_MODE, $post_data, 'ProductAdd', $product_id );

	}

	/**
	 * @param $product_id
	 *
	 * @return void
	 */
	public function Routee_update_product( $product_id ) {
		$product = wc_get_product( $product_id );
		$post_data = RouteeProductArray( $product );
		Routee_http_call( self::EVENT_MODE, $post_data, 'ProductUpdate', $product_id );
	}

	/**
	 * @param $product_id
	 *
	 * @return void
	 */
	public function Routee_trash_product( $product_id ) {
		$data['product_id'] = $product_id;
		Routee_http_call( self::EVENT_MODE, $data, 'ProductDelete', $product_id );
	}


	/**
	 * @param $product
	 *
	 * @return void
	 */
	public function Routee_woocommerce_no_stock( $product ) {
		$post_data = RouteeProductArray($product);
		$pro_data = $product->get_data();
		if($post_data['stock_quantity'] <= 0){
			if($pro_data['parent_id']!=0)
			{
				$data['product_id'] = $pro_data['parent_id'];
			}
			else
			{
				$data['product_id'] = $pro_data['id'];
			}
			Routee_http_call( self::EVENT_MODE, $data, 'ProductOutOfStock' );
		}
	}

	/**
	 * @param $product_id_with_stock
	 *
	 * @return void
	 */
	public function Routee_woocommerce_updated_product_stock( $product_id_with_stock ) {
		$WC_Product = wc_get_product( $product_id_with_stock );
		$post_data = RouteeProductArray($WC_Product);
		$variation_id = 0;//Stock update for product not for variation so should be 0
		$data['product_id'] = $product_id_with_stock;
		$data['variation_id'] = $variation_id;
		$data['stock_quantity'] = $post_data['stock_quantity'];
		Routee_http_call( self::EVENT_MODE, $data, 'ProductStockUpdate' );
	}

	/**
	 * @param $token
	 *
	 * @return void
	 */
	public function Routee_woocommerce_add_to_cart($token) {
		global $woocommerce;
		$items = $woocommerce->cart->get_cart();
		$cart_pro = array();
		foreach ($items as $key => $item) {
			$cart_pro[] = array(
				'product_id' => $item['product_id'],
				'variation_id' => $item['variation_id'],
				'quantity' => $item['quantity']
			);
		}
		$post_data = array(
			'customer_id' => get_current_user_id(),
			'cart_products' => $cart_pro,
		);
		Routee_http_call( self::EVENT_MODE, $post_data, 'CartUpdate', $token );
	}

	/**
	 * @param $cart_item_key
	 * @param $cart
	 *
	 * @return void
	 */
	public function Routee_remove_cart_items( $cart_item_key, $cart ) {
		$cart_pro = array();
		unset($cart->cart_contents[$cart_item_key]);
		if(!empty($cart->cart_contents)){
			foreach ($cart->cart_contents as $key => $value) {
				$cart_pro[] = array(
					'product_id' => $value['product_id'],
					'variation_id' => $value['variation_id'],
					'quantity' => $value['quantity'],
				);
			}
		}
		$post_data = array(
			'customer_id' => get_current_user_id(),
			'cart_products' => $cart_pro,
		);

		Routee_http_call( self::EVENT_MODE, $post_data, 'CartUpdate', $cart_item_key );
	}

	/**
	 * @param $order_id
	 *
	 * @return void
	 */
	public function Routee_woocommerce_checkout_order($order_id){
		$order = new WC_Order( $order_id );

		$pro_ary = array();
		foreach( $order->get_items() as $product ) {
			$pro_ary[] =array(
				'product_id' => $product['product_id'],
				'variation_id' => $product['variation_id'],
				'quantity'  =>  $product['qty']
			);
		}

		$shipping_method_title = '';
		foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
			$shipping_method_title     = $shipping_item_obj->get_method_title();
		}
		$order_data = $order->get_data();
		$user_details = RouteeUserDetails( $order_data );

		$post_data = array(
			'customer_id' =>  $order_data['customer_id'],
			'order_details' => array(
				'order_id' => $order_id,
				'order_status' => $order_data['status'],
				'shipping_method' => $shipping_method_title,
				'payment_method' => get_post_meta( $order_id, '_payment_method', true ),
				'total_price' => $order_data['total'],
				'lastname'=>  $user_details['customer']['lastname'],
				"firstname"=>  $user_details['customer']['firstname'],
				"email" => $user_details['customer']['email'],
				"phone" => $user_details['customer']['phone'],
				'addresses' => $user_details['addresses']
			),
			'order_products' => $pro_ary,
		);

		Routee_http_call( self::EVENT_MODE, $post_data, 'OrderAdd', $order_id );
	}

	/**
	 * @param $order_id
	 *
	 * @return void
	 */
	public function Routee_order_update_callback($order_id){
		$order = wc_get_order( $order_id );
		$order_data = $order->get_data();
		$tracking_number = '';
		$shipping_method_title = '';
		foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
			$shipping_method_title     = $shipping_item_obj->get_method_title();
		}
		$tracking_number = get_post_meta($order->get_order_number(), 'Tracking_ID', true);

		$post_data = array(
			'customer_id' =>  $order_data['customer_id'],
			'order_id' => $order_id,
			'order_status' => $order_data['status'],
			'shipping_method' => $shipping_method_title,
			'tracking_number' => $tracking_number
		);

		Routee_http_call( self::EVENT_MODE, $post_data, 'OrderStatusUpdate', $order_id );
	}

	/**
	 * @param $order_id
	 *
	 * @return void
	 */
	public function Routee_woocommerce_order_status_completed( $order_id ){
		$order = wc_get_order( $order_id );
		$data['customer_id'] = $order->get_user_id();
		$data['order_id'] = $order_id;
		Routee_http_call( self::EVENT_MODE, $data, 'OrderPaymentConfirmed', $order_id );
	}
}
