<?php

if ( ! defined( 'ABSPATH' ) ) exit;

class Routee_Http {

	const LOG_FILE_NAME = 'routee_log.log';

	private $routee_auth_url  = '';
	private $routee_data_url  = '';
	private $routee_event_url = '';
	private $try_count = 0;
	private $is_valid = 0;
	private $timeout = 0;

	public function __construct() {
		$this->set_urls();
	}

	private function set_urls() {
		$this->routee_auth_url  = ROUTEE_AUTH_URL;
		$this->routee_data_url  = ROUTEE_DATA_URL;
		$this->routee_event_url = ROUTEE_EVENT_URL;
	}

	/**
	 * @throws Exception
	 */
	private function post_data( $url, $data ) {
		$payload = json_encode( $data );
		$response = wp_remote_post($url, array(
				'method'      => 'POST',
				'timeout'     => $this->timeout,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => true,
				'headers'     => array(),
				'body'        => $payload
			)
		);
		$code = wp_remote_retrieve_response_code( $response );
		$body = json_decode( wp_remote_retrieve_body( $response ), TRUE );

		if ( is_wp_error( $response ) || $code != 200 ) {
			$msg = $body['message'] ?? 'Curl error : There was some error with http call.';
			if ( is_wp_error( $response ) ) {
				$code = $response->get_error_code();
				$msg = $response->get_error_message();
			}
			$code = is_int( $code ) ? $code : 0;
			error_log( $msg, 0 );
			throw new Exception( $msg, $code );
		} else {
			return array(
				'code' => $code,
				'body' => $body
			);
		}
	}

	private function prepare_data( $mode, $payload, $event ) {
		$data = $payload;
		if ( $mode === 'event' ){
			$token = get_option( 'RouteeLoginCode' );
			$data  = array( 'data' => $payload, 'uuid' => $token );
			if ( $event != '' ) {
				$data["event"] = $event;
			}
		}
		return $data;
	}

	private function get_url( $mode ) {
		$url = 'routee_' . $mode . '_url';
		return $this->{$url};
	}

	private function update_database( $data, $event, $post_id, $msg ) {
		global $wpdb;
		$table = $wpdb->prefix . 'events';
		$inID = "SELECT id FROM ".$table." WHERE event='{$event}' AND event_id='{$post_id}'";
		$nums = $wpdb->query($inID);
		$json = json_encode($data);
		if ($nums > 0) {
			$query = "UPDATE ".$table." SET req_msg='{$msg}',is_valid='{$this->is_valid}',updated_at=NOW(),json='{$json}' WHERE event='{$event}' AND event_id='{$post_id}'";
		} else {
			$query = "INSERT INTO ".$table." (event,event_id,req_msg,is_valid,created_at,json) VALUES ('{$event}','{$post_id}','{$msg}','{$this->is_valid}',NOW(),'{$json}')";
		}
		$wpdb->query($query);
	}

	private function write_log( $mode, $event, $exception ) {
		$message = "[" . date('Y-m-d H:i:s') . "] - Mode : " . $mode . ($event != "" ? ", Event : " . $event : "") .", Code : " . $exception->getCode() . ", Exception message : " . $exception->getMessage() . PHP_EOL;
		$file = plugin_dir_path( __FILE__ ) . self::LOG_FILE_NAME;
		file_put_contents( $file, $message, FILE_APPEND );
	}

	public function make_http_call( $mode, $payload, $event = '', $post_id = 0 ) {
		$this->try_count++;
		$this->timeout = $mode == 'event' ? 10 : 45;
		$data = $this->prepare_data( $mode, $payload, $event );
		$url = $this->get_url( $mode );
		try {
			$result = $this->post_data( $url, $data );
			$this->is_valid = 1;
 		}catch ( Exception $exception ) {
			$retry_codes = array( 429 );
			$this->is_valid = 0;
			$result = array(
				'code' => $exception->getCode(),
				'body' => array(
					'message' => $exception->getMessage()
				)
			);
			if ( in_array( $exception->getCode() , $retry_codes ) && $this->try_count < 3 ) {
				$this->make_http_call( $mode, $payload, $event, $post_id );
			} else {
				$this->write_log( $mode, $event, $exception );
			}
		}
		if ( $mode == 'event' ) {
			$this->update_database( $payload, $event, $post_id, $result['body']['message'] );
		}

		return array(
			'success' => $this->is_valid,
			'code' => $result['code'],
			'data' => $result['body']
		);
	}
}
